import { assertNotNull, defer } from "@tutao/tutanota-utils";
import { assertMainOrNodeBoot } from "../common/Env";
import { getWhitelabelCustomizations } from "../../../common/misc/WhitelabelCustomizations";
import { NotFoundError } from "../common/error/RestError";
import { client } from "../../misc/ClientDetector";
assertMainOrNodeBoot();
export class LoginController {
    constructor(loginFacade, loginListener) {
        this.loginFacade = loginFacade;
        this.loginListener = loginListener;
        this.userController = null;
        this.customizations = null;
        this.partialLogin = defer();
        this._isWhitelabel = !!getWhitelabelCustomizations(window);
        this.postLoginActions = [];
        this.fullyLoggedIn = false;
        this.atLeastPartiallyLoggedIn = false;
    }
    init() {
        this.waitForFullLogin().then(async () => {
            this.fullyLoggedIn = true;
            await this.waitForPartialLogin();
            for (const lazyAction of this.postLoginActions) {
                const action = await lazyAction();
                await action.onFullLoginSuccess({
                    sessionType: this.getUserController().sessionType,
                    userId: this.getUserController().userId,
                });
            }
        });
    }
    /**
     * create a new session and set up stored credentials and offline database, if applicable.
     * @param username the mail address being used to log in
     * @param password the password given to log in
     * @param sessionType whether to store the credentials in local storage
     * @param databaseKey if given, will use this key for the offline database. if not, will force a new database to be created and generate a key.
     */
    async createSession(username, password, sessionType, databaseKey = null) {
        const newSessionData = await this.loginFacade.createSession(username, password, client.getIdentifier(), sessionType, databaseKey);
        const { user, credentials, sessionId, userGroupInfo } = newSessionData;
        await this.onPartialLoginSuccess({
            user,
            userGroupInfo,
            sessionId,
            accessToken: credentials.accessToken,
            sessionType,
            loginUsername: username,
        }, sessionType);
        return newSessionData;
    }
    addPostLoginAction(handler) {
        this.postLoginActions.push(handler);
    }
    async onPartialLoginSuccess(initData, sessionType) {
        const { initUserController } = await import("./UserController");
        this.userController = await initUserController(initData);
        await this.loadCustomizations();
        await this._determineIfWhitelabel();
        for (const lazyHandler of this.postLoginActions) {
            const handler = await lazyHandler();
            await handler.onPartialLoginSuccess({
                sessionType,
                userId: initData.user._id,
            });
        }
        this.atLeastPartiallyLoggedIn = true;
        this.partialLogin.resolve();
    }
    async createExternalSession(userId, password, salt, kdfType, clientIdentifier, sessionType) {
        const persistentSession = sessionType === 2 /* SessionType.Persistent */;
        const { user, credentials, sessionId, userGroupInfo } = await this.loginFacade.createExternalSession(userId, password, salt, kdfType, clientIdentifier, persistentSession);
        await this.onPartialLoginSuccess({
            user,
            accessToken: credentials.accessToken,
            sessionType,
            sessionId,
            userGroupInfo,
            loginUsername: userId,
        }, 0 /* SessionType.Login */);
        return credentials;
    }
    /**
     * Resume an existing session using stored credentials, may or may not unlock a persistent local database
     * @param unencryptedCredentials The stored credentials and optional database key for the offline db
     * @param externalUserKeyDeriver The KDF type and salt to resume a session
     * @param offlineTimeRangeDays the user configured time range for their offline storage, used to initialize the offline db
     */
    async resumeSession(unencryptedCredentials, externalUserKeyDeriver, offlineTimeRangeDays) {
        const { unencryptedToCredentials } = await import("../../misc/credentials/Credentials.js");
        const credentials = unencryptedToCredentials(unencryptedCredentials);
        const resumeResult = await this.loginFacade.resumeSession(credentials, externalUserKeyDeriver ?? null, unencryptedCredentials.databaseKey ?? null, offlineTimeRangeDays ?? null);
        if (resumeResult.type === "error") {
            return resumeResult;
        }
        else {
            const { user, userGroupInfo, sessionId } = resumeResult.data;
            await this.onPartialLoginSuccess({
                user,
                accessToken: credentials.accessToken,
                userGroupInfo,
                sessionId,
                sessionType: 2 /* SessionType.Persistent */,
                loginUsername: credentials.login,
            }, 2 /* SessionType.Persistent */);
            return { type: "success" };
        }
    }
    isUserLoggedIn() {
        return this.userController != null;
    }
    isFullyLoggedIn() {
        return this.fullyLoggedIn;
    }
    isAtLeastPartiallyLoggedIn() {
        return this.atLeastPartiallyLoggedIn;
    }
    waitForPartialLogin() {
        return this.partialLogin.promise;
    }
    async waitForFullLogin() {
        // Full login event might be received before we finish userLogin on the client side because they are done in parallel.
        // So we make sure to wait for userLogin first.
        await this.waitForPartialLogin();
        const loginListener = await this.loginListener();
        return loginListener.waitForFullLogin();
    }
    isInternalUserLoggedIn() {
        return this.isUserLoggedIn() && this.getUserController().isInternalUser();
    }
    isGlobalAdminUserLoggedIn() {
        return this.isUserLoggedIn() && this.getUserController().isGlobalAdmin();
    }
    getUserController() {
        return assertNotNull(this.userController); // only to be used after login (when user is defined)
    }
    isEnabled(feature) {
        return this.customizations != null ? this.customizations.indexOf(feature) !== -1 : false;
    }
    loadCustomizations() {
        if (this.isInternalUserLoggedIn()) {
            return this.getUserController()
                .loadCustomer()
                .then((customer) => {
                this.customizations = customer.customizations.map((f) => f.feature);
            });
        }
        else {
            return Promise.resolve();
        }
    }
    async logout(sync) {
        if (this.userController) {
            await this.userController.deleteSession(sync);
            this.userController = null;
            this.partialLogin = defer();
            this.fullyLoggedIn = false;
            const loginListener = await this.loginListener();
            loginListener.reset();
            this.init();
        }
        else {
            console.log("No session to delete");
        }
    }
    async _determineIfWhitelabel() {
        this._isWhitelabel = await this.getUserController().isWhitelabelAccount();
    }
    isWhitelabel() {
        return this._isWhitelabel;
    }
    /**
     * Deletes the session on the server.
     * @param credentials
     * @param pushIdentifier identifier associated with this device, if any, to delete PushIdentifier on the server
     */
    async deleteOldSession(credentials, pushIdentifier = null) {
        try {
            await this.loginFacade.deleteSession(credentials.accessToken, pushIdentifier);
        }
        catch (e) {
            if (e instanceof NotFoundError) {
                console.log("session already deleted");
            }
            else {
                throw e;
            }
        }
    }
    async retryAsyncLogin() {
        const loginListener = await this.loginListener();
        loginListener.onRetryLogin();
        await this.loginFacade.retryAsyncLogin();
    }
}
//# sourceMappingURL=LoginController.js.map